window.Gleam.initSearch([{"doc":"string_width","title":"string_width","content":"# string_width\n### A low-ish level library for building terminal UIs.\n\n[![Package Version](https://img.shields.io/hexpm/v/string_width)](https://hex.pm/packages/string_width)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/string_width/)\n\n```sh\ngleam add string_width@3\n```\n\n`string_width` provides functions to measure the required amount of cells, and then layout strings in an ANSI-aware manner. It includes ready-to-use layout functions like [limit](./string_width.html#limit &quot;Word wrapping and truncation&quot;) or [stack_horizontal](./string_width.html#stack_horizontal &quot;Build column-based layouts&quot;), as well as low-level primitives, allowing you to build your own layout algorithms.\n\nAll Gleam targets are supported, and it passes the tests of the NPM [string-width](https://www.npmjs.com/package/string-width) and [unicode-width](https://crates.io/crates/unicode-width) crate where applicable. Compared to others, a heavy focus is put on on reporting the actual width of strings in terminals instead (Check out [Internals](./internals.html) for more information.)\n\nIt is also one of the fastest options available, even including target-specific ones, while still providing you full flexibility and correctness.\n\n### Tour\n\n```gleam\nimport gleam/int\nimport gleam/io\nimport gleam/list\nimport gleam/result\nimport string_width.{Left, Size, Top}\n\n// we will render a simple help menu for a command-line app here.\n// the help menu will automatically adjust based on the width of the terminal.\n\ntype Command {\n  Command(name: String, description: String)\n}\n\nconst commands = [\n  Command(\n    name: &quot;build&quot;,\n    description: &quot;Build and bundle your application.\\nThe generated bundle calls your apps&#39; main function. If your main function accepts a single argument of type List(String), all additional command-line arguments will be passed there.&quot;,\n  ),\n  Command(\n    name: &quot;dev&quot;,\n    description: &quot;Start a file watcher that automatically recompiles your app on all file changes, and then re-runs your tests. Compilation errors or test failures will be displayed in an overlay, and if everything succeeds, your window will automatically hot-reload.&quot;,\n  ),\n  Command(\n    name: &quot;help&quot;,\n    description: &quot;Show this help text.&quot;\n  ),\n]\n\npub fn main() {\n  // get the size of the terminal, or fallback to a default size.\n  let term_size =\n    string_width.get_terminal_size()\n    |&gt; result.unwrap(Size(rows: 24, columns: 80))\n\n  // how big does the left-hand side need to be?\n  let left_width =\n    list.fold(commands, 0, fn(max, cmd) {\n      int.max(max, string_width.line(cmd.name))\n    })\n\n  // add some gap between both columns and compute the size of the right side.\n  let gap = 4\n  // set a maximum width of 60 for the right column to improve readability.\n  let right_width = int.min(60, term_size.columns - left_width - gap)\n\n  commands\n  |&gt; list.map(fn(cmd) {\n    [\n      cmd.name\n        |&gt; pink\n        // make sure all left-hand sides are padded to left_width\n        |&gt; string_width.align(left_width, Left, with: &quot; &quot;),\n      // word wrap the right-hand side such that it fits into our column.\n      // if the description would overflow 10 lines, truncate it.\n      cmd.description\n        |&gt; string_width.limit(\n          to: Size(rows: 10, columns: right_width),\n          ellipsis: &quot;…&quot;,\n        )\n        // make sure that if we used styles those would wrap properly\n        |&gt; string_width.inline_styles,\n    ]\n    // build the column layout;\n    // the name should be aligned with the top of the description\n    |&gt; string_width.stack_horizontal(place: Top, gap:, with: &quot; &quot;)\n  })\n  // combine the help text of all commands, adding a line of space in between.\n  |&gt; string_width.stack_vertical(align: Left, gap: 1, with: &quot; &quot;)\n  |&gt; io.println\n}\n\n// string_width is agnostic to the way you add styles to your strings.\n// maybe check out gleam_community_ansi!\nfn pink(str: String) {\n  &quot;\\u{1b}[38;5;207m&quot; &lt;&gt; str &lt;&gt; &quot;\\u{1b}[m&quot;\n}\n```\n\nOutput:\n\n&lt;div style=&quot;\n  white-space:pre;\n  margin:var(--gap) 0;\n  border-radius:1px;\n  overflow:auto;\n  box-shadow:var(--shadow);\n  font-family:&#39;Ubuntu Mono&#39;, SFMono-Regular, &#39;SF Mono&#39;, Menlo, Consolas, &#39;Liberation Mono&#39;, ui-monospace, monospace;\n  line-height:1.2;\n  background-color:var(--code-background);\n  padding:1em;\n&quot;&gt;&lt;span style=&quot;color:#ff5fff&quot;&gt;build&lt;/span&gt;    Build and bundle your application.\n         The generated bundle calls your apps&#39; main function. If your\n         main function accepts a single argument of type\n         List(String), all additional command-line arguments will be\n         passed there.&lt;br&gt;\n&lt;span style=&quot;color:#ff5fff&quot;&gt;dev&lt;/span&gt;      Start a file watcher that automatically recompiles your app\n         on all file changes, and then re-runs your tests.\n         Compilation errors or test failures will be displayed in an\n         overlay, and if everything succeeds, your window will\n         automatically hot-reload.&lt;br&gt;\n&lt;span style=&quot;color:#ff5fff&quot;&gt;help&lt;/span&gt;     Show this help text.\n&lt;/div&gt;\n\n### Measuring strings\n\n```gleam\n  string_width.dimensions(&quot;hello,\\n안녕하세요&quot;)\n  // --&gt; string_width.Size(rows: 2, columns: 10)\n\n  string_width.line_with(\n    &quot;👩‍👩‍👦‍👦&quot;,\n    string_width.new() |&gt; string_width.mode_2027,\n  )\n  // --&gt; 2\n\n  // word wrapping and truncation\n  string_width.limit(\n    &quot;Lorem ipsum dolor sit amet\\nIs a common placeholder string&quot;,\n    to: Size(rows: 3, columns: 10),\n    ellipsis: &quot;...&quot;\n  )\n  // --&gt; &quot;Lorem ipsum dolor\\nsit amet\\nIs a common place...&quot;\n\n  // position a string inside a box\n  string_width.position(\n    &quot;XXX&quot;,\n    in: Size(rows: 3, columns: 10),\n    align: Center,\n    place: Middle,\n    with: &quot;.&quot;\n  )\n  // --&gt; &quot;..........\\n...XXX....\\n..........&quot;\n}\n```\n\n\n### Limitations\n\n- Many terminals don&#39;t support grapheme clusters properly, so by default, the\n  values returned by this library try to match the behaviour of libc instead\n  of modern text rendering pipelines.\n\n  If you encounter a mismatch that is consistent across multiple environments,\n  feel free to open an issue or ping me on Discord!\n\n- The behaviour when encountering stray variant selectors does not match other similar libraries.\n\n\n### Sources\n\nThe table lookup technique used by this library is heavily based on the musl libc `wcwidth` implementation. I built updated tables using the Unicode 16.0 data, and added support for ambiguous characters. It also uses the regex of the ansi-regex npm package, and the test cases of the string-width npm package.\n\n- **Grapheme Clusters in Terminals:** [https://mitchellh.com/writing/grapheme-clusters-in-terminals](https://mitchellh.com/writing/grapheme-clusters-in-terminals)\n- **UAX #11 East Asian Width:** [https://www.unicode.org/reports/tr11/](https://www.unicode.org/reports/tr11/)\n- **Terminal Unicode Core**: [https://github.com/contour-terminal/terminal-unicode-core](https://github.com/contour-terminal/terminal-unicode-core)\n- **string-width:** [https://github.com/sindresorhus/string-width](https://github.com/sindresorhus/string-width)\n- **musl-libc wcwidth:** [https://git.musl-libc.org/cgit/musl/tree/src/ctype/wcwidth.c](https://git.musl-libc.org/cgit/musl/tree/src/ctype/wcwidth.c)\n- **reflow:** [https://github.com/muesli/reflow](https://github.com/muesli/reflow)\n","url":"index.html"},{"doc":"string_width","title":"string_width","content":"# Comparison\n\nCompared to many other libraries in this space, the main focus of this one is\nmeasuring the real width of things when printed to terminal emulators. This\nlibrary does **not** try to guess what font shaping (especially ligatures) would\ndo, and cares **less** about the way things look like in other monospace contexts,\nsuch as using a modern rendering pipeline (e.g. in a browser) but with a monospace\nfont. I encourage you to use your font rendering library directly, or find a way\nto layout things without needing to know the size of your text.\n\n# Unicode version\n\nThe data files are taken from the Unicode 16 release (Sep 2024).\n\n# Algorithm\n\nComponents (codepoints, control sequences and grapheme clusters) are classified\ninto 4 catgories:\n\n- **zero-width:** these do not advance the current column\n- **narrow:** these advance the cursor by one column\n- **wide:** these advance the cursor by two columns\n- **ambiguous:** these advance the cursor either by one or two columns, depending\n  on the `ambiguous_as_wide` option.\n\nControl characters and ansi escape sequences are ignored and reported as zero-width,\n**except**:\n\n - Newlines (`\\n` and `\\r\\n`)  increase the current row, while resetting the column.\n - Tabs (`\\t`) increase the current column to the next tab stop.\n\nFor the remaining string, 3 different modes are supported: `wcwidth`, `mode_2027`\nand `mode2027_ext`.\n\n## Default mode (`wcwidth`)\n\nBy default, this library tries to match the output of the `wcwidth` function from\nglibc. This function is still widely used by the majority of terminal emulators.\n\nIn this mode, the width of a string is the sum of the widths of its individual\ncode points. (Extended) grapheme cluster boundaries are ignored.\n\n## (Extended) Mode 2027\n\n_Mode 2027_ is a proposed mode for terminal emulators that applications can\nrequest. When active and supported, the terminal emulator is supposed to follow\nthe [terminal unicode core spec](https://github.com/contour-terminal/terminal-unicode-core).\nThis mode is supported by some terminals, and even the default and only\nbehaviour in some others.\n\nNone of the terminals I tested implement the proposed spec fully, and the exact\nbehaviour is subject to ongoing discussion in the freedesktop.org terminal\nworking group. Support for this mode should therefore be considered best-effort\nat the moment, especially when support for Brahmic, Arabic, and some east asian\nscripts is required. If you just want your emoji family to be 2 columns wide,\nthis mode works well enough right now.\n\nThis library does the following:\n\n- Extended grapheme clusters are segmented based on the `pop_grapheme` function.\n  This function uses native target code, and is therefore not guaranteed to match\n  the unicode version of this library (at the time of writing,\n  Erlang uses 15.0, and Node.JS uses 15.1).\n- The width of a grapheme cluster is the _maximum_ width of all codepoints\n  that make it up.\n- If a grapheme cluster contains U+FE0F Variant Selector 16 in addition to\n  another non-zero-width codepoint, its width is wide.\n\nAdditionally, in the extended mode, the following rules apply:\n\n- If a grapheme cluster contains two or more non-spacing non-zero-width\n  codepoints, its width is wide. &quot;Spacing&quot; codepoints in this context are all\n  codepoints with a `General_Category` of `Spacing_Mark`.\n\nThis rule makes the width reported by this library across Erlang and Nodejs more\nstable (Unicode version differences, see above), as well as closer match the\nbehaviour of actual terminals with mode 2027 support. Starting with Unicode 15.1,\nsome sequences that ocupy multiple columns are now segmented into single\ngrapheme clusters.\n\n\n## Width of a single codepoint\n\nThe following codepoints are classified as **zero-with:**\n\n- All codepoints with the `Default_Ignorable_Code_Point` property, **except** for U+115F Hangul Choseong Filler.\n- All codepoints with a `General_Category` of `Control`, `Enclosing_Mark`, `Nonspacing_Mark`, `Paragraph_Separator`, or `Line_Separator`.\n- All codepoints with `Hangul_Syllable_Type` of Vowel or Trailing Jamo.\n- All codepoints marked as `Prepended_Concatenation_Mark`\n- `U+fff9..U+fffb`, interlinear annotation format characters\n- `U+13430..U+13440`, egyptian hieroglyph format characters\n\nThe following codepoints are classified as **wide:**\n\n- All codepoints with the `Emoji_Presentation` property set.\n- All codepoints with an `East_Asian_Width` of Wide or Fullwidth.\n\nThe following codepoints are classified as **ambiguous:**\n\n- All codepoints with an `East_Asian_Width` of Ambiguous.\n\nAll other codepoints are **narrow**.\n\n# Testing\n\nI currently test against VTE (mainly gnome-terminal), Windows Terminal, Kitty,\nand foot for mode 2027 support. I also test on Contour since they originally\nproposed the mode 2027 spec; however, they use a custom Unicode library that I\ndon&#39;t trust fully.\n\nWhen reporting a mismatch, please include which terminal and the escaped\ncodepoints. Gitlab/Discord sometimes like to strip certain modifiers.\n","url":"internals.html"},{"doc":"string_width","title":"Alignment","content":"pub type Alignment {\n  Left\n  Center\n  Right\n}\n Control the text or box alignment on the horizontal axis.\nLeft\n\n\nCenter\n\n\nRight\n\n\nSynonyms:\nstring_width.Alignment\nstring_width Alignment","url":"string_width.html#Alignment"},{"doc":"string_width","title":"Options","content":"pub opaque type Options\n Options to change the default behaviour of the functions in this library.\n If you are unsure what to do here, the defaults should work great!\n\nSynonyms:\nstring_width.Options\nstring_width Options","url":"string_width.html#Options"},{"doc":"string_width","title":"Piece","content":"pub type Piece {\n  Piece(piece: String, row: Int, column: Int, width: Int)\n}\n The measured pieces of a string, passed to you by `fold` and `fold_with`\nPiece(piece: String, row: Int, column: Int, width: Int)\n\n\nSynonyms:\nstring_width.Piece\nstring_width Piece","url":"string_width.html#Piece"},{"doc":"string_width","title":"Placement","content":"pub type Placement {\n  Top\n  Middle\n  Bottom\n}\n Control the box alignment on the vertical axis.\nTop\n\n\nMiddle\n\n\nBottom\n\n\nSynonyms:\nstring_width.Placement\nstring_width Placement","url":"string_width.html#Placement"},{"doc":"string_width","title":"Size","content":"pub type Size {\n  Size(rows: Int, columns: Int)\n}\n A custom type making it easy to differentiate the width (columns) and\n height (rows).\nSize(rows: Int, columns: Int)\n\n\nSynonyms:\nstring_width.Size\nstring_width Size","url":"string_width.html#Size"},{"doc":"string_width","title":"align","content":"pub fn align(\n  str: String,\n  to max_width: Int,\n  align alignment: Alignment,\n  with space: String,\n) -&gt; String\n Align each line horizontally, using a spacer character or string as a filler.\n Each line of the return value will be at least `max_width` wide.\n\n If a line is already longer than the max width, it will not be changed.\n If the space strings width does not evenly divide the missing amount of\n columns, the extra spacer will overflow the max width.\n\n ### Examples\n\n \n align(&quot;hello&quot;, to: 10, align: Left, with: &quot; &quot;)\n // --&gt; &quot;hello     &quot;\n\n align(&quot;12345&quot;, to: 8, align: Right, with: &quot;0&quot;)\n // --&gt; &quot;00012345&quot;\n\n align(&quot; Welcome &quot;, to: 20, align: Center, with: &quot;==&quot;)\n // --&gt; &quot;====== Welcome ======&quot; // (len = 21)\n\n align(&quot;Trans\\nrights\\nare\\nhuman\\nrights&quot;, to: 7, align: Right, with: &quot; &quot;)\n // --&gt; &quot;  Trans\\n rights\\n    are\\n  human\\n rights&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.align\nstring_width align","url":"string_width.html#align"},{"doc":"string_width","title":"align_with","content":"pub fn align_with(\n  str: String,\n  to max_width: Int,\n  align alignment: Alignment,\n  using options: Options,\n  with space: String,\n) -&gt; String\n Like `align`, bu use custom options for measure.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.align_with\nstring_width align_with","url":"string_width.html#align_with"},{"doc":"string_width","title":"ambiguous_as_wide","content":"pub fn ambiguous_as_wide(options: Options) -&gt; Options\n Some characters are marked by Unicode as &quot;ambiguous&quot;, meaning they may\n occupy 1 or 2 cells, depending on the context, current language, selected\n font, surrounding text, and more.\n\n Unicode recommends treating these characters as narrow by default,\n but you can change this behaviour using this option.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.ambiguous_as_wide\nstring_width ambiguous_as_wide","url":"string_width.html#ambiguous_as_wide"},{"doc":"string_width","title":"at_tab_offset","content":"pub fn at_tab_offset(\n  options: Options,\n  tab_offset: Int,\n) -&gt; Options\n Define an _offset_ for tab stop calculations. (Default: 0)\n\n If the text you print doesn&#39;t start at the first column, but is instead\n indented somehow, you can set this option to this number to make sure tab\n stops are correctly calculated.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.at_tab_offset\nstring_width at_tab_offset","url":"string_width.html#at_tab_offset"},{"doc":"string_width","title":"count_ansi_codes","content":"pub fn count_ansi_codes(options: Options) -&gt; Options\n Do not ignore ansi escape sequences, and count them as regular characters.\n\n You can enable this option as an optimisation if you are sure that your\n string doesn&#39;t contain any ansi escape codes.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.count_ansi_codes\nstring_width count_ansi_codes","url":"string_width.html#count_ansi_codes"},{"doc":"string_width","title":"dimensions","content":"pub fn dimensions(str: String) -&gt; Size\n The required number of rows and columns to display the given text.\n\n The number of rows is equal to the number of lines, while the number of\n columns represents the maximum line width.\n\n ### Examples\n\n \n dimensions(&quot;안녕하세요&quot;)\n // --&gt; Size(rows: 1, columns: 10)\n\n dimensions(&quot;hello,\\n안녕하세요&quot;)\n // --&gt; Size(rows: 2, columns: 10)\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.dimensions\nstring_width dimensions","url":"string_width.html#dimensions"},{"doc":"string_width","title":"dimensions_with","content":"pub fn dimensions_with(str: String, options: Options) -&gt; Size\n Like `dimensions`, but use custom options.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.dimensions_with\nstring_width dimensions_with","url":"string_width.html#dimensions_with"},{"doc":"string_width","title":"fold","content":"pub fn fold(\n  over str: String,\n  from state: a,\n  with fun: fn(a, Piece) -&gt; a,\n) -&gt; a\n A higher-level `fold` that keeps track of the position inside of the string.\n\n Handles tabs and newlines, and always passes full grapheme clusters,\n regardless of options. Concatenating the graphemes produces the original string.\n\n Intended to be used as a basis for custom layout algorithms.\n\n ### Example\n\n \n fold(&quot;hi 😊&quot;, from: [], with: list.prepend)\n |&gt; list.reverse\n // --&gt; [Piece(&quot;h&quot;, 0, 0, 1), Piece(&quot;i&quot;, 0, 1, 1), Piece(&quot; &quot;, 0, 2, 1), Piece(&quot;😊&quot;, 0, 3, 2)]\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.fold\nstring_width fold","url":"string_width.html#fold"},{"doc":"string_width","title":"fold_raw","content":"pub fn fold_raw(\n  over string: String,\n  using options: Options,\n  from state: a,\n  with fun: fn(a, String, Int) -&gt; a,\n) -&gt; a\n A low-level fold that iterate over the measured components of a string.\n Components are either graphemes or codepoints depending on the `mode` option,\n or other undivisible sequences, like ANSI escape codes.\n\n This function does not by itself keep track of any additional state, and\n doesn&#39;t for example handle newlines or tabs like `fold` would.\n If you know that your algorithm is safe and won&#39;t break up graphemes (or\n maybe this is acceptable), you can use this fold variant instead as a\n performance optimisation.\n\n Concatenating all components is guaranteed to produce the original string.\n\n ### Examples\n\n \n // A slower string_width.line implementation that doesn&#39;t handle tabs\n fold_raw(&quot;hello&quot;, new(), from: 0, with: fn(so_far, _chr, width) { width + so_far })\n // --&gt; 5\n \n\n \n // truncate a string after 50 characters, but keep all ansi sequences.\n let options = new()\n use #(total, acc), chr, width &lt;- fold_raw(input, options, from: #(0, &quot;&quot;))\n case total &gt;= 50 {\n   True -&gt; case is_ansi_component(chr, options) {\n     True -&gt; #(total, acc &lt;&gt; chr)\n     False -&gt; #(total, acc)\n   }\n   False -&gt; #(total + width, acc &lt;&gt; chr)\n }\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.fold_raw\nstring_width fold_raw","url":"string_width.html#fold_raw"},{"doc":"string_width","title":"fold_raw_pieces","content":"pub fn fold_raw_pieces(\n  over string: String,\n  using options: Options,\n  from state: a,\n  with fun: fn(a, Piece) -&gt; a,\n) -&gt; a\n A version of `fold_raw` that does keep track of some state for you to\n handle tabs, and count the current row/column, passing `Piece` values to\n you instead.\n\n ### Example\n\n \n fold_raw_pieces(&quot;hi!&quot;, using: new(), from: &quot;&quot;, with: list.prepend)\n |&gt; list.reverse\n // --&gt; [Piece(&quot;h&quot;, 0, 0, 1), Piece(&quot;i&quot;, 0, 1, 1), Piece(&quot;!&quot;, 0, 2, 1)]\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.fold_raw_pieces\nstring_width fold_raw_pieces","url":"string_width.html#fold_raw_pieces"},{"doc":"string_width","title":"fold_with","content":"pub fn fold_with(\n  over str: String,\n  using options: Options,\n  from state: a,\n  with fun: fn(a, Piece) -&gt; a,\n) -&gt; a\n A higher-level `fold` that keeps track of the position inside of the string.\n Like `fold`, but using custom options to measure the pieces.\n\n Handles tabs and newlines, and always passes full grapheme clusters,\n regardless of options. Concatenating the graphemes produces the original string.\n\n Intended to be used as a basis for custom layout algorithms.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.fold_with\nstring_width fold_with","url":"string_width.html#fold_with"},{"doc":"string_width","title":"get_terminal_size","content":"pub fn get_terminal_size() -&gt; Result(Size, Nil)\n Get the size of the terminal, if available.\n\n Checks the built-in methods for the target, as well as the `LINES` and\n `COLUMNS` environment variables.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.get_terminal_size\nstring_width get_terminal_size","url":"string_width.html#get_terminal_size"},{"doc":"string_width","title":"inline_styles","content":"pub fn inline_styles(str: String) -&gt; String\n Make sure [SGR ansi codes](https://en.wikipedia.org/wiki/ANSI_escape_code#SGR_(Select_Graphic_Rendition)_parameters)\n (these are the ones you&#39;d use for colors and styling!) never wrap\n around a line. This makes it save to use other layout functions on the\n resulting string, without affecting the styling anymore.\n\n It does so by collecting all SGR sequences since the last reset, and\n re-applying them on each line break.\n\n **Warning:** This function assumes it is safe to emit reset commands,\n and that no ambient styling is in effect (that is, the terminal would be in\n its default state when printing the resulting string!)\n\n ### Examples\n\n \n // &quot;\\u{1b}[31m&quot;: red text color\n // &quot;\\u{1b}[m&quot;: reset/normal mode\n inline_styles(&quot;\\u{1b}[31mhello\\nworld&quot;)\n // --&gt; &quot;\\u{1b}[31mhello&quot; &lt;&gt; &quot;\\u{1b}[m\\n&quot; &lt;&gt; &quot;\\u{1b}[31mworld&quot; &lt;&gt; &quot;\\u{1b}[m&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.inline_styles\nstring_width inline_styles","url":"string_width.html#inline_styles"},{"doc":"string_width","title":"is_ansi_component","content":"pub fn is_ansi_component(str: String, options: Options) -&gt; Bool\n Returns true if a given component string recieved in `fold` is an ANSI\n escape sequence.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.is_ansi_component\nstring_width is_ansi_component","url":"string_width.html#is_ansi_component"},{"doc":"string_width","title":"limit","content":"pub fn limit(\n  str: String,\n  to max_size: Size,\n  ellipsis ellipsis: String,\n) -&gt; String\n Limit the dimensions of a string, either by wrapping on white space\n characters or soft hyphens, or by truncating the last line and appending an\n ellipsis.\n\n The lines of the resulting string are left aligned and are at most `columns`\n wide. If a single word is longer than the maximum allowed line width, the\n word is broken into 2 pieces at the grapheme level. If the string gets\n truncated, this function will keep collecting ANSI sequences to make sure\n all formatting is preserved.\n\n A commonly used ellipsis character is `&quot;…&quot;`, also known as\n U+2026 HORIZONTAL ELLIPSIS.\n\n ### Examples\n\n \n limit(&quot;Hello World&quot;, Size(rows: 1, columns: 10), ellipsis: &quot;...&quot;)\n // --&gt; &quot;Hello W...&quot;\n\n limit(&quot;Hello World&quot;, Size(rows: 2, columns: 10), ellipsis: &quot;...&quot;)\n // --&gt; &quot;Hello\\nWorld&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.limit\nstring_width limit","url":"string_width.html#limit"},{"doc":"string_width","title":"limit_with","content":"pub fn limit_with(\n  str: String,\n  to max_size: Size,\n  using options: Options,\n  ellipsis ellipsis: String,\n) -&gt; String\n Like `limit`, but also customise the options used for measuring.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.limit_with\nstring_width limit_with","url":"string_width.html#limit_with"},{"doc":"string_width","title":"line","content":"pub fn line(str: String) -&gt; Int\n Get the number of columns required to print a line in a terminal.\n\n If multiple lines are given, the length of the longest line is returned.\n\n ### Examples\n\n \n line(&quot;äöüè&quot;)\n // --&gt; 4\n\n line(&quot;안녕하세요&quot;)\n // --&gt; 10\n\n line(&quot;👩‍👩‍👦‍👦&quot;)\n // --&gt; 8\n\n line(&quot;\\u{1B}[31mhello\\u{1B}[39m&quot;)\n // --&gt; 5\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.line\nstring_width line","url":"string_width.html#line"},{"doc":"string_width","title":"line_with","content":"pub fn line_with(str: String, options: Options) -&gt; Int\n Like `line`, but use custom options.\n\n ### Example\n\n \n let options =\n   new()\n   |&gt; mode_2027\n\n line_with(&quot;👩‍👩‍👦‍👦&quot;, options)\n // --&gt; 2\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.line_with\nstring_width line_with","url":"string_width.html#line_with"},{"doc":"string_width","title":"mode_2027","content":"pub fn mode_2027(options: Options) -&gt; Options\n Measure grapheme clusters instead of individual code points.\n\n Most terminal emulators do not handle grapheme clusters well and will\n instead show their decomposition. To make sure a given string always fits\n even on those terminals, the functions in this package will copy this\n behaviour as well.\n\n If you enable this option, the returned numbers will more accurately represent\n the width of a string on a website or in an editor.\n\n **NOTE:** Grapheme support in terminals is highly experimental and subject\n to ongoing discussion! When working on CLIs or TUIs, you do not want to use\n this option most of the time.\n\n See also [Grapheme Clusters and Terminal Emulators](https://mitchellh.com/writing/grapheme-clusters-in-terminals)\n for a better explanation on how terminals behave.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.mode_2027\nstring_width mode_2027","url":"string_width.html#mode_2027"},{"doc":"string_width","title":"mode_2027_ext","content":"pub fn mode_2027_ext(options: Options) -&gt; Options\n Measure grapheme clusters instead of individual code points, and always\n treat grapheme clusters with more than one non-modifier character as wide.\n\n This is a custom extension to Unicode Core. Many single grapheme clusters\n would currently still render as wide glyphs in many contexts, since font\n shaping would not be able to collapse them into a single narrow &quot;ligature&quot;.\n\n This mode is an additional heuristic that tries to handle more such cases.\n\n See `mode_2027` for more explanation on the implications.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.mode_2027_ext\nstring_width mode_2027_ext","url":"string_width.html#mode_2027_ext"},{"doc":"string_width","title":"mode_wcwidth","content":"pub fn mode_wcwidth(options: Options) -&gt; Options\n The default mode and the mode suitable for most terminals.\n\n Measures individual code points, similar to the libc `wcwidth` function\n that almost all of these terminals use.\n\n Note that except for `fold_raw`, this library will still always process\n grapheme clusters as a unit.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.mode_wcwidth\nstring_width mode_wcwidth","url":"string_width.html#mode_wcwidth"},{"doc":"string_width","title":"new","content":"pub fn new() -&gt; Options\n Start building up new options.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.new\nstring_width new","url":"string_width.html#new"},{"doc":"string_width","title":"padding","content":"pub fn padding(\n  str: String,\n  top top: Int,\n  right right: Int,\n  bottom bottom: Int,\n  left left: Int,\n  with space: String,\n) -&gt; String\n Add some padding around the box of a string, filling empty space and the\n padded area with the provided character.\n\n Padding is given as a number of rows/columns. Values are given clockwise,\n following the CSS shorthand property.\n\n Ansi sequences will always be kept. If the space strings&#39; width does not\n evenly divide the missing area in the computed outer size of the string,\n the extra spacer will the resulting lines may not all be equally long.\n\n **Tip:** Provide negative values to drop columns or rows!\n\n ### Examples\n\n \n padding(&quot;o&quot;, 1, 1, 1, 1, with: &quot;x&quot;)\n // --&gt; &quot;xxx\\nxox\\nxxx&quot;\n\n padding(&quot;hello&quot;, 0, 0, 0, right: -1, with: &quot;&quot;)\n // --&gt; &quot;hell&quot;\n\n padding(&quot;Trans\\nrights!&quot;, top: 2, right: 5, bottom: 1, left: 1, with: &quot;-&quot;)\n // --&gt; &quot;-------------\\n-------------\\n-Trans-------\\n-rights!-----\\n-------------&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.padding\nstring_width padding","url":"string_width.html#padding"},{"doc":"string_width","title":"padding_with","content":"pub fn padding_with(\n  str: String,\n  top top: Int,\n  right right: Int,\n  bottom bottom: Int,\n  left left: Int,\n  using options: Options,\n  with space: String,\n) -&gt; String\n Like `padding`, but use custom options for measuring.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.padding_with\nstring_width padding_with","url":"string_width.html#padding_with"},{"doc":"string_width","title":"position","content":"pub fn position(\n  str: String,\n  in bounding_box: Size,\n  align alignment: Alignment,\n  place placement: Placement,\n  with space: String,\n) -&gt; String\n Position the string area inside a bigger box without changing text alignment.\n The box will be filled with the space character.\n\n **Tip:** If you only want to align a string on one axis, you can provide\n `0` for the orthogonal one! Extra characters and lines will be kept.\n\n If the space strings&#39; width does not evenly divide the missing amount of\n columns, the extra spacer will overflow the max width.\n\n ### Examples\n\n \n position(&quot;X&quot;, in: Size(3, 3), align: Center, place: Middle, with: &quot;o&quot;)\n // --&gt; &quot;ooo\\noXo\\nooo&quot;\n\n // text still left-aligned, but moved to the right by 1 space\n position(&quot;Trans\\nrights\\nare\\nhuman\\nrights&quot;, Size(0, 7), Right, Top, &quot; &quot;)\n // --&gt; &quot; Trans \\n rights\\n are   \\n human \\n rights&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.position\nstring_width position","url":"string_width.html#position"},{"doc":"string_width","title":"position_with","content":"pub fn position_with(\n  str: String,\n  in bounding_box: Size,\n  horizontal alignment: Alignment,\n  vertical placement: Placement,\n  using options: Options,\n  with space: String,\n) -&gt; String\n Like `position`, but use custom options to measure each line.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.position_with\nstring_width position_with","url":"string_width.html#position_with"},{"doc":"string_width","title":"scroll","content":"pub fn scroll(\n  str: String,\n  top top: Int,\n  left left: Int,\n  view viewport_size: Size,\n  with space: String,\n) -&gt; String\n Cut out a `viewort` at the given origin `#(top, left)` from a string. This\n is similar to _&quot;scrolling&quot;_ a web page.\n\n You can scroll past the filled area of the string in any direction.\n Empty space is filled using the `space` string. If the space string does not\n evenly fill the empty area, some lines may be longer than the given viewport.\n\n All ANSI sequences in the entire string will be kept, even if they are\n outside of the visible viewport.\n\n ### Example\n\n \n scroll(&quot;wibble&quot;, top: 0, left: 2, view: Size(1, 2), with: &quot; &quot;)\n // --&gt; &quot;bb&quot;\n\n &quot;Hello\\nHow are you today?\\nDid you know:\\nLucy says trans rights!&quot;\n |&gt; scroll(top: 2, left: 10, view: Size(3, 5), with: &quot;-&quot;)\n // --&gt; &quot;ow:--\\ntrans\\n-----&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.scroll\nstring_width scroll","url":"string_width.html#scroll"},{"doc":"string_width","title":"scroll_with","content":"pub fn scroll_with(\n  str: String,\n  top top: Int,\n  left left: Int,\n  view viewport_size: Size,\n  using options: Options,\n  with space: String,\n) -&gt; String\n Like `scroll`, but provide custom options for measuring.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.scroll_with\nstring_width scroll_with","url":"string_width.html#scroll_with"},{"doc":"string_width","title":"soft_wrap","content":"pub fn soft_wrap(\n  str: String,\n  to max_size: Size,\n  ellipsis ellipsis: String,\n  to_indent to_indent: fn(String) -&gt; String,\n) -&gt; String\n Limit the dimensions of a string, either by wrapping on white space\n characters or soft hyphens while using a custom indentation function, or by\n truncating the last line and appending an ellipsis.\n\n This function gives a little bit more control over what happens on soft\n line breaks than `limit` and allows you to inspect the line printed so far\n to figure out the indentation for all soft-wrapped lines. The `to_indent`\n callback is called with the line so far, and should return an indentation\n string (usually just the proper amount of spaces) that is prepended to all\n the following soft-wrapped lines.\n\n Behaves like `limit` otherwise.\n\n ### Examples\n\n \n fn to_indent(str) {\n   case str {\n     // if it is a list, indent with 2 spaces\n     &quot;- &quot; &lt;&gt; _ -&gt; &quot;  &quot;\n     // if it is a blockquote, prepend another blockquote marker\n     &quot;&gt; &quot; &lt;&gt; _ -&gt; &quot;&gt; &quot;\n     // else, don&#39;t indent (like limit)\n     _ -&gt; &quot;&quot;\n   }\n }\n\n // Girly Girl Productions - 10 drunk cigarettes\n let lyrics = &quot;\n &gt; Getting girls rich, yeah, that&#39;s a part of my plan.\n &gt; And I could name 10 things us girls need, before we ever need a man:\n\n - One new vape, two lines of coke\n - Three drinks from the bar, four more lines of coke\n - Five Guys fries, six hits from my blunt\n - Seven more lines of coke, eight pairs of shoes\n - Nine BB belts, and 10 drunk cigarettes.\n &quot;\n\n lyrics\n |&gt; soft_wrap(to: Size(rows: 24, columns: 34), ellipsis: &quot;...&quot;, to_indent:)\n |&gt; io.println\n \n\n **Output:**\n\n txt\n &gt; Getting girls rich, yeah, that&#39;s\n &gt; a part of my plan.\n &gt; And I could name 10 things us\n &gt; girls need, before we ever need\n &gt; a man:\n\n - One new vape, two lines of coke\n - Three drinks from the bar, four\n   more lines of coke\n - Five Guys fries, six hits from\n   my blunt\n - Seven more lines of coke, eight\n   pairs of shoes\n - Nine BB belts, and 10 drunk\n   cigarettes.\n \n\n **Note:** This is not meant to allow for arbitrary text formatting, but as a\n quick way to indicate soft-wrapping in the printed text. As shown in the\n example, it _can_ be used to do some very light formatting, but you should\n almost always parse the string first, and print it in a way that makes sense\n for your text format, for example using the awesome\n [glam](https://hexdocs.pm/glam/) package!\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.soft_wrap\nstring_width soft_wrap","url":"string_width.html#soft_wrap"},{"doc":"string_width","title":"soft_wrap_with","content":"pub fn soft_wrap_with(\n  str: String,\n  to max_size: Size,\n  using options: Options,\n  ellipsis ellipsis: String,\n  to_indent to_indent: fn(String) -&gt; String,\n) -&gt; String\n Like `soft_wrap`, but using custom options to measure the strings and indents.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.soft_wrap_with\nstring_width soft_wrap_with","url":"string_width.html#soft_wrap_with"},{"doc":"string_width","title":"stack_horizontal","content":"pub fn stack_horizontal(\n  blocks: List(String),\n  place vertical: Placement,\n  gap gap: Int,\n  with space: String,\n) -&gt; String\n Stack multiple blocks of strings horizontally next to each other into\n multiple columns.\n\n The resulting string will as high as the highest string in the list. All\n other blocks can be `position`-ed vertically relative to this size.\n\n **Tip;** Use [limit](#limit) and [inline_styles](#inline_styles) to lay out\n the text in your columns!\n\n ### Example\n\n \n [\n   &quot;--color&quot;,\n   &quot;Enable color support.\\nThis option is enabled by default in a terminal.&quot;\n ]\n |&gt; stack_horizontal(place: Top, gap: 4, with: &quot; &quot;)\n // --&gt; &quot;--color    Enable color support.                           \\n&quot;\n //  &lt;&gt; &quot;           This option is enabled by default in a terminal.&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.stack_horizontal\nstring_width stack_horizontal","url":"string_width.html#stack_horizontal"},{"doc":"string_width","title":"stack_horizontal_with","content":"pub fn stack_horizontal_with(\n  blocks: List(String),\n  place vertical: Placement,\n  gap gap: Int,\n  using options: Options,\n  with space: String,\n) -&gt; String\n Like `stack_horizontal`, but using custom options for measure.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.stack_horizontal_with\nstring_width stack_horizontal_with","url":"string_width.html#stack_horizontal_with"},{"doc":"string_width","title":"stack_vertical","content":"pub fn stack_vertical(\n  blocks: List(String),\n  align horizontal: Alignment,\n  gap gap: Int,\n  with space: String,\n) -&gt; String\n Stack multiple blocks of strings on top of each other into multiple rows.\n\n The resulting string will as wide as the widest string in the list. All other\n blocks can be `position`-ed horizontally relative to this size.\n\n ### Example\n\n \n [\n   &quot;Hello!&quot;,\n   &quot;I hope you are feeling fantastic today!&quot;\n ]\n |&gt; stack_vertical(Center, gap: 1, with: &quot; &quot;)\n // --&gt; &quot;                Hello!                 \\n&quot;\n //  &lt;&gt; &quot;                                       \\n&quot;\n //  &lt;&gt; &quot;I hope you are feeling fantastic today!&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.stack_vertical\nstring_width stack_vertical","url":"string_width.html#stack_vertical"},{"doc":"string_width","title":"stack_vertical_with","content":"pub fn stack_vertical_with(\n  blocks: List(String),\n  align horizontal: Alignment,\n  gap gap: Int,\n  using options: Options,\n  with space: String,\n) -&gt; String\n Like `stack_vertical`, but using custom options for measure.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.stack_vertical_with\nstring_width stack_vertical_with","url":"string_width.html#stack_vertical_with"},{"doc":"string_width","title":"strip_ansi","content":"pub fn strip_ansi(str: String) -&gt; String\n Strip _all_ ansi sequences from a string, using the same matching algorithm\n that this library uses internally.\n\n This makes it safe to enable `count_ansi_codes`, or print a string without\n having to worry that it might mess with the terminal and/or colors.\n\n Unexpected characters inside of escape sequences are ignored (swallowed).\n\n ### Examples\n\n \n strip_ansi(&quot;\\u{1b}[0;33;49;3;9;4mhi~\\u{1b}[0m&quot;)\n // --&gt; &quot;hi~&quot;\n\n strip_ansi(&quot;check out \\u{1b}]8;;https://gleam.run\\u{7}Gleam!\\u{1b}]8;;\\u{7}&quot;)\n // --&gt; &quot;check out Gleam!&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.strip_ansi\nstring_width strip_ansi","url":"string_width.html#strip_ansi"},{"doc":"string_width","title":"tabs_to_spaces","content":"pub fn tabs_to_spaces(str: String) -&gt; String\n Replace all tab characters found in the string with the amount of spaces\n that this tab would have had otherwise.\n\n This makes it safe to prepend to a line, without changing the spacing produced\n by tabs anymore.\n\n ### Examples\n\n \n tabs_to_spaces(&quot;Hello\\tWorld&quot;)\n // --&gt; &quot;Hello   World&quot; // 3 spaces\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.tabs_to_spaces\nstring_width tabs_to_spaces","url":"string_width.html#tabs_to_spaces"},{"doc":"string_width","title":"tabs_to_spaces_with","content":"pub fn tabs_to_spaces_with(\n  str: String,\n  using options: Options,\n) -&gt; String\n Like `tabs_to_spaces`, but customise the opttions as well.\n\n **NB:** You can use `at_tab_offset` and `with_tab_width` to change the measure of\n tab characters inside the string!\n\n \n let options = new() |&gt; with_tab_width(4)\n tabs_to_spaces_with(&quot;hi\\tcutie~&quot;, options)\n // --&gt; &quot;hi  cutie~&quot;\n \n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.tabs_to_spaces_with\nstring_width tabs_to_spaces_with","url":"string_width.html#tabs_to_spaces_with"},{"doc":"string_width","title":"with_tab_width","content":"pub fn with_tab_width(\n  options: Options,\n  tab_width: Int,\n) -&gt; Options\n Change the number of columns between tab stops. (Default: 8)\n\n Whenever a tab character `\\t` is encountered, the current column number will\n be rounded up to the next multiple of this number.\n\n &lt;div style=&quot;text-align: right;&quot;&gt;\n     &lt;a href=&quot;#&quot;&gt;\n         &lt;small&gt;Back to top ↑&lt;/small&gt;\n     &lt;/a&gt;\n &lt;/div&gt;\nSynonyms:\nstring_width.with_tab_width\nstring_width with_tab_width","url":"string_width.html#with_tab_width"},{"doc":"string_width","title":"string_width","content":" &lt;style&gt;\n   small {\n     font-size: 0.7em;\n     opacity: 0.75;\n   }\n &lt;/style&gt;\n\n The `with` variants of all functions additionally work with custom options.\n\n **Tip:** Hover the links for short summaries!\n\n #### Measure\n [line](#line &quot;Width of a line&quot;)&lt;small&gt;[[_with](#line_with)]&lt;/small&gt;\n [dimensions](#dimensions &quot;Size of a string&quot;)&lt;small&gt;[[_with](#dimensions_with)]&lt;/small&gt;,\n [get_terminal_size](#get_terminal_size &quot;Size of the terminal window&quot;)\n\n #### Layout &amp; Positioning\n\n [limit](#limit &quot;Word wrapping and truncation&quot;)&lt;small&gt;[[_with](#limit_with)]&lt;/small&gt;,\n [soft_wrap](#soft_wrap &quot;Word wrapping with custom indentation logic&quot;)&lt;small&gt;[[_with](#soft_wrap_with)]&lt;/small&gt;,\n [align](#align &quot;Text alignment&quot;)&lt;small&gt;[[_with](#align_with)]&lt;/small&gt;,\n [tabs_to_spaces](#tabs_to_spaces &quot;Pin tab widths&quot;)&lt;small&gt;[[_with](#tabs_to_spaces_with)]&lt;/small&gt;\n\n [stack_horizontal](#stack_horizontal &quot;Column-based layouts&quot;)&lt;small&gt;[[_with](#stack_horizontal_with)]&lt;/small&gt;,\n [stack_vertical](#stack_vertical &quot;Row-based layouts&quot;)&lt;small&gt;[[_with](#stack_vertical_with)]&lt;/small&gt;,\n [padding](#padding &quot;Add extra space around a string&quot;)&lt;small&gt;[[_with](#padding_with)]&lt;/small&gt;,\n [position](#position &quot;Put a string inside a fixed-size box&quot;)&lt;small&gt;[[_with](#position_with)]&lt;/small&gt;,\n [scroll](#scroll &quot;Cutout a viewport area from a string&quot;)&lt;small&gt;[[_with](#scroll_with)]&lt;/small&gt;\n\n #### ANSI escape sequence helpers\n [inline_styles](#inline_styles &quot;Ensure styling doesn&#39;t overflow lines/columns&quot;),\n [strip_ansi](#strip_ansi &quot;Remove all styles&quot;),\n [is_ansi_component](#is_ansi_component &quot;Helper for fold&quot;)\n\n #### Options Builder\n [new](#new &quot;Default options&quot;),\n [ambiguous_as_wide](#ambiguous_as_wide &quot;CJK support&quot;),\n [count_ansi_codes](#count_ansi_codes &quot;Non-terminal support&quot;),\n [mode_2027](#mode_2027 &quot;Compatibility with other libraries&quot;),\n [mode_2027_ext](#mode_2027_ext &quot;Compatibility with other libraries with extra salt&quot;),\n [mode_wcwidth](#mode_wcwidth &quot;Default mode, use this for terminals!&quot;),\n [at_tab_offset](#at_tab_offset &quot;Control tab width calculations&quot;),\n [with_tab_width](#with_tab_width &quot;Control tab width calculations&quot;)\n\n #### Advanced\n [fold](#fold &quot;Loop grapheme clusters and their position/size&quot;)&lt;small&gt;[[_with](#fold_with)]&lt;/small&gt;,\n [fold_raw](#fold_raw &quot;Loop low-level segments and their un-processed width&quot;),\n [fold_raw_pieces](#fold_raw_pieces &quot;Loop low-level segments and their position/size&quot;)\n","url":"string_width.html"}]);